/// <reference path="jquery.js" />
/*! Copyright (c) 2011 Brandon Aaron (http://brandonaaron.net)
 * Licensed under the MIT License (LICENSE.txt).
 *
 * Thanks to: http://adomas.org/javascript-mouse-wheel/ for some pointers.
 * Thanks to: Mathias Bank(http://www.mathias-bank.de) for a scope bug fix.
 * Thanks to: Seamus Leahy for adding deltaX and deltaY
 *
 * Version: 3.0.6
 *
 * Requires: 1.2.2+
 */

(function ($) {

	var types = ['DOMMouseScroll', 'mousewheel'];

	if ($.event.fixHooks) {
		for (var i = types.length; i;) {
			$.event.fixHooks[types[--i]] = $.event.mouseHooks;
		}
	}

	$.event.special.mousewheel = {
		setup: function () {
			if (this.addEventListener) {
				for (var i = types.length; i;) {
					this.addEventListener(types[--i], handler, false);
				}
			} else {
				this.onmousewheel = handler;
			}
		},

		teardown: function () {
			if (this.removeEventListener) {
				for (var i = types.length; i;) {
					this.removeEventListener(types[--i], handler, false);
				}
			} else {
				this.onmousewheel = null;
			}
		}
	};

	$.fn.extend({
		mousewheel: function (fn) {
			return fn ? this.bind("mousewheel", fn) : this.trigger("mousewheel");
		},

		unmousewheel: function (fn) {
			return this.unbind("mousewheel", fn);
		}
	});


	function handler(event) {
		var orgEvent = event || window.event, args = [].slice.call(arguments, 1), delta = 0, returnValue = true, deltaX = 0, deltaY = 0;
		event = $.event.fix(orgEvent);
		event.type = "mousewheel";

		// Old school scrollwheel delta
		if (orgEvent.wheelDelta) { delta = orgEvent.wheelDelta / 120; }
		if (orgEvent.detail) { delta = -orgEvent.detail / 3; }

		// New school multidimensional scroll (touchpads) deltas
		deltaY = delta;

		// Gecko
		if (orgEvent.axis !== undefined && orgEvent.axis === orgEvent.HORIZONTAL_AXIS) {
			deltaY = 0;
			deltaX = -1 * delta;
		}

		// Webkit
		if (orgEvent.wheelDeltaY !== undefined) { deltaY = orgEvent.wheelDeltaY / 120; }
		if (orgEvent.wheelDeltaX !== undefined) { deltaX = -1 * orgEvent.wheelDeltaX / 120; }

		// Add event and delta to the front of the arguments
		args.unshift(event, delta, deltaX, deltaY);

		return ($.event.dispatch || $.event.handle).apply(this, args);
	}

})(jQuery);


/**
 * @license 
 * jQuery Tools @VERSION Tooltip - UI essentials
 * 
 * NO COPYRIGHTS OR LICENSES. DO WHAT YOU LIKE.
 * 
 * http://flowplayer.org/tools/tooltip/
 *
 * Since: November 2008
 * Date: @DATE 
 */
(function($) { 	
	// static constructs
	$.tools = $.tools || {version: '@VERSION'};
	
	$.tools.tooltip = {
		
		conf: { 
			
			// default effect variables
			effect: 'toggle',			
			fadeOutSpeed: "fast",
			predelay: 0,
			delay: 30,
			opacity: 1,			
			tip: 0,
            fadeIE: false, // enables fade effect in IE
			
			// 'top', 'bottom', 'right', 'left', 'center'
			position: ['top', 'center'], 
			offset: [0, 0],
			relative: false,
			cancelDefault: true,
			
			// type to event mapping 
			events: {
				def: 			"mouseenter,mouseleave",
				input: 		"focus,blur",
				widget:		"focus mouseenter,blur mouseleave",
				tooltip:		"mouseenter,mouseleave"
			},
			
			// 1.2
			layout: '<div/>',
			tipClass: 'tooltip'
		},
		
		addEffect: function(name, loadFn, hideFn) {
			effects[name] = [loadFn, hideFn];	
		} 
	};
	
	
	var effects = { 
		toggle: [ 
			function(done) { 
				var conf = this.getConf(), tip = this.getTip(), o = conf.opacity;
				if (o < 1) { tip.css({opacity: o}); }
				tip.show();
				done.call();
			},
			
			function(done) { 
				this.getTip().hide();
				done.call();
			} 
		],
		
		fade: [
			function(done) {
				var conf = this.getConf();
				if (!$.browser.msie || conf.fadeIE) {
					this.getTip().fadeTo(conf.fadeInSpeed, conf.opacity, done);
				}
				else {
					this.getTip().show();
					done();
				}
			},
			function(done) {
				var conf = this.getConf();
				if (!$.browser.msie || conf.fadeIE) {
					this.getTip().fadeOut(conf.fadeOutSpeed, done);
				}
				else {
					this.getTip().hide();
					done();
				}
			}
		]		
	};   

		
	/* calculate tip position relative to the trigger */  	
	function getPosition(trigger, tip, conf) {	

		
		// get origin top/left position 
		var top = conf.relative ? trigger.position().top : trigger.offset().top, 
			 left = conf.relative ? trigger.position().left : trigger.offset().left,
			 pos = conf.position[0];

		top  -= tip.outerHeight() - conf.offset[0];
		left += trigger.outerWidth() + conf.offset[1];
		
		// iPad position fix
		if (/iPad/i.test(navigator.userAgent)) {
			top -= $(window).scrollTop();
		}
		
		// adjust Y		
		var height = tip.outerHeight() + trigger.outerHeight();
		if (pos == 'center') 	{ top += height / 2; }
		if (pos == 'bottom') 	{ top += height; }
		
		
		// adjust X
		pos = conf.position[1]; 	
		var width = tip.outerWidth() + trigger.outerWidth();
		if (pos == 'center') 	{ left -= width / 2; }
		if (pos == 'left')   	{ left -= width; }	 
		
		return {top: top, left: left};
	}		

	
	
	function Tooltip(trigger, conf) {

		var self = this, 
			 fire = trigger.add(self),
			 tip,
			 timer = 0,
			 pretimer = 0, 
			 title = trigger.attr("title"),
			 tipAttr = trigger.attr("data-tooltip"),
			 effect = effects[conf.effect],
			 shown,
				 
			 // get show/hide configuration
			 isInput = trigger.is(":input"), 
			 isWidget = isInput && trigger.is(":checkbox, :radio, select, :button, :submit"),			
			 type = trigger.attr("type"),
			 evt = conf.events[type] || conf.events[isInput ? (isWidget ? 'widget' : 'input') : 'def']; 
		
		
		// check that configuration is sane
		if (!effect) { throw "Nonexistent effect \"" + conf.effect + "\""; }					
		
		evt = evt.split(/,\s*/); 
		if (evt.length != 2) { throw "Tooltip: bad events configuration for " + type; } 
		
		
		// trigger --> show  
		trigger.on(evt[0], function(e) {

			clearTimeout(timer);
			if (conf.predelay) {
				pretimer = setTimeout(function() { self.show(e); }, conf.predelay);	
				
			} else {
				self.show(e);	
			}
			
		// trigger --> hide
		}).on(evt[1], function(e)  {
			clearTimeout(pretimer);
			if (conf.delay)  {
				timer = setTimeout(function() { self.hide(e); }, conf.delay);	
				
			} else {
				self.hide(e);		
			}
			
		}); 
		
		
		// remove default title
		if (title && conf.cancelDefault) { 
			trigger.removeAttr("title");
			trigger.data("title", title);			
		}		
		
		$.extend(self, {
				
			show: function(e) {  

				// tip not initialized yet
				if (!tip) {
					
					// data-tooltip 
					if (tipAttr) {
						tip = $(tipAttr);

					// single tip element for all
					} else if (conf.tip) { 
						tip = $(conf.tip).eq(0);
						
					// autogenerated tooltip
					} else if (title) { 
						tip = $(conf.layout).addClass(conf.tipClass).appendTo(document.body)
							.hide().append(title);

					// manual tooltip
					} else {	
						tip = trigger.next();  
						if (!tip.length) { tip = trigger.parent().next(); } 	 
					}
					
					if (!tip.length) { throw "Cannot find tooltip for " + trigger;	}
				} 
			 	
			 	if (self.isShown()) { return self; }  
				
			 	// stop previous animation
			 	tip.stop(true, true); 			 	
			 	
				// get position
				var pos = getPosition(trigger, tip, conf);			
		
				// restore title for single tooltip element
				if (conf.tip) {
					tip.html(trigger.data("title"));
				}

				// onBeforeShow
				e = $.Event();
				e.type = "onBeforeShow";
				fire.trigger(e, [pos]);				
				if (e.isDefaultPrevented()) { return self; }
		
				
				// onBeforeShow may have altered the configuration
				pos = getPosition(trigger, tip, conf);
				
				// set position
				tip.css({position:'absolute', top: pos.top, left: pos.left});					
				
				shown = true;
				
				// invoke effect 
				effect[0].call(self, function() {
					e.type = "onShow";
					shown = 'full';
					fire.trigger(e);		 
				});					

	 	
				// tooltip events       
				var event = conf.events.tooltip.split(/,\s*/);

				if (!tip.data("__set")) {
					
					tip.off(event[0]).on(event[0], function() { 
						clearTimeout(timer);
						clearTimeout(pretimer);
					});
					
					if (event[1] && !trigger.is("input:not(:checkbox, :radio), textarea")) { 					
						tip.off(event[1]).on(event[1], function(e) {
	
							// being moved to the trigger element
							if (e.relatedTarget != trigger[0]) {
								trigger.trigger(evt[1].split(" ")[0]);
							}
						}); 
					} 
					
					// bind agein for if same tip element
					if (!conf.tip) tip.data("__set", true);
				}
				
				return self;
			},
			
			hide: function(e) {

				if (!tip || !self.isShown()) { return self; }
			
				// onBeforeHide
				e = $.Event();
				e.type = "onBeforeHide";
				fire.trigger(e);				
				if (e.isDefaultPrevented()) { return; }
	
				shown = false;
				
				effects[conf.effect][1].call(self, function() {
					e.type = "onHide";
					fire.trigger(e);		 
				});
				
				return self;
			},
			
			isShown: function(fully) {
				return fully ? shown == 'full' : shown;	
			},
				
			getConf: function() {
				return conf;	
			},
				
			getTip: function() {
				return tip;	
			},
			
			getTrigger: function() {
				return trigger;	
			}		

		});		

		// callbacks	
		$.each("onHide,onBeforeShow,onShow,onBeforeHide".split(","), function(i, name) {
				
			// configuration
			if ($.isFunction(conf[name])) { 
				$(self).on(name, conf[name]); 
			}

			// API
			self[name] = function(fn) {
				if (fn) { $(self).on(name, fn); }
				return self;
			};
		});
		
	}
		
	
	// jQuery plugin implementation
	$.fn.tooltip = function(conf) {
		
		// return existing instance
		var api = this.data("tooltip");
		if (api) { return api; }

		conf = $.extend(true, {}, $.tools.tooltip.conf, conf);
		
		// position can also be given as string
		if (typeof conf.position == 'string') {
			conf.position = conf.position.split(/,?\s/);	
		}
		
		// install tooltip for each entry in jQuery object
		this.each(function() {
			api = new Tooltip($(this), conf); 
			$(this).data("tooltip", api); 
		});
		
		return conf.api ? api: this;		 
	};
		
}) (jQuery);

/* http://keith-wood.name/timeEntry.html
   Time entry for jQuery v1.4.9.
   Written by Keith Wood (kbwood{at}iinet.com.au) June 2007.
   Dual licensed under the GPL (http://dev.jquery.com/browser/trunk/jquery/GPL-LICENSE.txt) and
   MIT (http://dev.jquery.com/browser/trunk/jquery/MIT-LICENSE.txt) licenses.
   Please attribute the author if you use it. */
(function ($) { function TimeEntry() { this._disabledInputs = []; this.regional = []; this.regional[''] = { show24Hours: false, separator: ':', ampmPrefix: '', ampmNames: ['AM', 'PM'], spinnerTexts: ['Now', 'Previous field', 'Next field', 'Increment', 'Decrement'] }; this._defaults = { appendText: '', showSeconds: false, timeSteps: [1, 1, 1], initialField: 0, useMouseWheel: true, defaultTime: null, minTime: null, maxTime: null, spinnerImage: 'spinnerDefault.png', spinnerSize: [20, 20, 8], spinnerBigImage: '', spinnerBigSize: [40, 40, 16], spinnerIncDecOnly: false, spinnerRepeat: [500, 250], beforeShow: null, beforeSetTime: null }; $.extend(this._defaults, this.regional['']) } var m = 'timeEntry'; $.extend(TimeEntry.prototype, { markerClassName: 'hasTimeEntry', setDefaults: function (a) { extendRemove(this._defaults, a || {}); return this }, _connectTimeEntry: function (b, c) { var d = $(b); if (d.hasClass(this.markerClassName)) { return } var e = {}; e.options = $.extend({}, c); e._selectedHour = 0; e._selectedMinute = 0; e._selectedSecond = 0; e._field = 0; e.input = $(b); $.data(b, m, e); var f = this._get(e, 'spinnerImage'); var g = this._get(e, 'spinnerText'); var h = this._get(e, 'spinnerSize'); var i = this._get(e, 'appendText'); var j = (!f ? null : $('<span class="timeEntry_control" style="display: inline-block; ' + 'background: url(\'' + f + '\') 0 0 no-repeat; ' + 'width: ' + h[0] + 'px; height: ' + h[1] + 'px;' + ($.browser.mozilla && $.browser.version < '1.9' ? ' padding-left: ' + h[0] + 'px; padding-bottom: ' + (h[1] - 18) + 'px;' : '') + '"></span>')); d.wrap('<span class="timeEntry_wrap"></span>').after(i ? '<span class="timeEntry_append">' + i + '</span>' : '').after(j || ''); d.addClass(this.markerClassName).bind('focus.timeEntry', this._doFocus).bind('blur.timeEntry', this._doBlur).bind('click.timeEntry', this._doClick).bind('keydown.timeEntry', this._doKeyDown).bind('keypress.timeEntry', this._doKeyPress); if ($.browser.mozilla) { d.bind('input.timeEntry', function (a) { $.timeEntry._parseTime(e) }) } if ($.browser.msie) { d.bind('paste.timeEntry', function (a) { setTimeout(function () { $.timeEntry._parseTime(e) }, 1) }) } if (this._get(e, 'useMouseWheel') && $.fn.mousewheel) { d.mousewheel(this._doMouseWheel) } if (j) { j.mousedown(this._handleSpinner).mouseup(this._endSpinner).mouseover(this._expandSpinner).mouseout(this._endSpinner).mousemove(this._describeSpinner) } }, _enableTimeEntry: function (a) { this._enableDisable(a, false) }, _disableTimeEntry: function (a) { this._enableDisable(a, true) }, _enableDisable: function (b, c) { var d = $.data(b, m); if (!d) { return } b.disabled = c; if (b.nextSibling && b.nextSibling.nodeName.toLowerCase() == 'span') { $.timeEntry._changeSpinner(d, b.nextSibling, (c ? 5 : -1)) } $.timeEntry._disabledInputs = $.map($.timeEntry._disabledInputs, function (a) { return (a == b ? null : a) }); if (c) { $.timeEntry._disabledInputs.push(b) } }, _isDisabledTimeEntry: function (a) { return $.inArray(a, this._disabledInputs) > -1 }, _changeTimeEntry: function (a, b, c) { var d = $.data(a, m); if (d) { if (typeof b == 'string') { var e = b; b = {}; b[e] = c } var f = this._extractTime(d); extendRemove(d.options, b || {}); if (f) { this._setTime(d, new Date(0, 0, 0, f[0], f[1], f[2])) } } $.data(a, m, d) }, _destroyTimeEntry: function (b) { $input = $(b); if (!$input.hasClass(this.markerClassName)) { return } $input.removeClass(this.markerClassName).unbind('.timeEntry'); if ($.fn.mousewheel) { $input.unmousewheel() } this._disabledInputs = $.map(this._disabledInputs, function (a) { return (a == b ? null : a) }); $input.parent().replaceWith($input); $.removeData(b, m) }, _setTimeTimeEntry: function (a, b) { var c = $.data(a, m); if (c) { if (b === null || b === '') { c.input.val('') } else { this._setTime(c, b ? (typeof b == 'object' ? new Date(b.getTime()) : b) : null) } } }, _getTimeTimeEntry: function (a) { var b = $.data(a, m); var c = (b ? this._extractTime(b) : null); return (!c ? null : new Date(0, 0, 0, c[0], c[1], c[2])) }, _getOffsetTimeEntry: function (a) { var b = $.data(a, m); var c = (b ? this._extractTime(b) : null); return (!c ? 0 : (c[0] * 3600 + c[1] * 60 + c[2]) * 1000) }, _doFocus: function (a) { var b = (a.nodeName && a.nodeName.toLowerCase() == 'input' ? a : this); if ($.timeEntry._lastInput == b || $.timeEntry._isDisabledTimeEntry(b)) { $.timeEntry._focussed = false; return } var c = $.data(b, m); $.timeEntry._focussed = true; $.timeEntry._lastInput = b; $.timeEntry._blurredInput = null; var d = $.timeEntry._get(c, 'beforeShow'); extendRemove(c.options, (d ? d.apply(b, [b]) : {})); $.data(b, m, c); $.timeEntry._parseTime(c); setTimeout(function () { $.timeEntry._showField(c) }, 10) }, _doBlur: function (a) { $.timeEntry._blurredInput = $.timeEntry._lastInput; $.timeEntry._lastInput = null }, _doClick: function (b) { var c = b.target; var d = $.data(c, m); if (!$.timeEntry._focussed) { var e = $.timeEntry._get(d, 'separator').length + 2; d._field = 0; if (c.selectionStart != null) { for (var f = 0; f <= Math.max(1, d._secondField, d._ampmField) ; f++) { var g = (f != d._ampmField ? (f * e) + 2 : (d._ampmField * e) + $.timeEntry._get(d, 'ampmPrefix').length + $.timeEntry._get(d, 'ampmNames')[0].length); d._field = f; if (c.selectionStart < g) { break } } } else if (c.createTextRange) { var h = $(b.srcElement); var i = c.createTextRange(); var j = function (a) { return { thin: 2, medium: 4, thick: 6 }[a] || a }; var k = b.clientX + document.documentElement.scrollLeft - (h.offset().left + parseInt(j(h.css('border-left-width')), 10)) - i.offsetLeft; for (var f = 0; f <= Math.max(1, d._secondField, d._ampmField) ; f++) { var g = (f != d._ampmField ? (f * e) + 2 : (d._ampmField * e) + $.timeEntry._get(d, 'ampmPrefix').length + $.timeEntry._get(d, 'ampmNames')[0].length); i.collapse(); i.moveEnd('character', g); d._field = f; if (k < i.boundingWidth) { break } } } } $.data(c, m, d); $.timeEntry._showField(d); $.timeEntry._focussed = false }, _doKeyDown: function (a) { if (a.keyCode >= 48) { return true } var b = $.data(a.target, m); switch (a.keyCode) { case 9: return (a.shiftKey ? $.timeEntry._changeField(b, -1, true) : $.timeEntry._changeField(b, +1, true)); case 35: if (a.ctrlKey) { $.timeEntry._setValue(b, '') } else { b._field = Math.max(1, b._secondField, b._ampmField); $.timeEntry._adjustField(b, 0) } break; case 36: if (a.ctrlKey) { $.timeEntry._setTime(b) } else { b._field = 0; $.timeEntry._adjustField(b, 0) } break; case 37: $.timeEntry._changeField(b, -1, false); break; case 38: $.timeEntry._adjustField(b, +1); break; case 39: $.timeEntry._changeField(b, +1, false); break; case 40: $.timeEntry._adjustField(b, -1); break; case 46: $.timeEntry._setValue(b, ''); break } return false }, _doKeyPress: function (a) { var b = String.fromCharCode(a.charCode == undefined ? a.keyCode : a.charCode); if (b < ' ') { return true } var c = $.data(a.target, m); $.timeEntry._handleKeyPress(c, b); return false }, _doMouseWheel: function (a, b) { if ($.timeEntry._isDisabledTimeEntry(a.target)) { return } b = ($.browser.opera ? -b / Math.abs(b) : ($.browser.safari ? b / Math.abs(b) : b)); var c = $.data(a.target, m); c.input.focus(); if (!c.input.val()) { $.timeEntry._parseTime(c) } $.timeEntry._adjustField(c, b); a.preventDefault() }, _expandSpinner: function (b) { var c = $.timeEntry._getSpinnerTarget(b); var d = $.data($.timeEntry._getInput(c), m); if ($.timeEntry._isDisabledTimeEntry(d.input[0])) { return } var e = $.timeEntry._get(d, 'spinnerBigImage'); if (e) { d._expanded = true; var f = $(c).offset(); var g = null; $(c).parents().each(function () { var a = $(this); if (a.css('position') == 'relative' || a.css('position') == 'absolute') { g = a.offset() } return !g }); var h = $.timeEntry._get(d, 'spinnerSize'); var i = $.timeEntry._get(d, 'spinnerBigSize'); $('<div class="timeEntry_expand" style="position: absolute; left: ' + (f.left - (i[0] - h[0]) / 2 - (g ? g.left : 0)) + 'px; top: ' + (f.top - (i[1] - h[1]) / 2 - (g ? g.top : 0)) + 'px; width: ' + i[0] + 'px; height: ' + i[1] + 'px; background: transparent url(' + e + ') no-repeat 0px 0px; z-index: 10;"></div>').mousedown($.timeEntry._handleSpinner).mouseup($.timeEntry._endSpinner).mouseout($.timeEntry._endExpand).mousemove($.timeEntry._describeSpinner).insertAfter(c) } }, _getInput: function (a) { return $(a).siblings('.' + $.timeEntry.markerClassName)[0] }, _describeSpinner: function (a) { var b = $.timeEntry._getSpinnerTarget(a); var c = $.data($.timeEntry._getInput(b), m); b.title = $.timeEntry._get(c, 'spinnerTexts')[$.timeEntry._getSpinnerRegion(c, a)] }, _handleSpinner: function (a) { var b = $.timeEntry._getSpinnerTarget(a); var c = $.timeEntry._getInput(b); if ($.timeEntry._isDisabledTimeEntry(c)) { return } if (c == $.timeEntry._blurredInput) { $.timeEntry._lastInput = c; $.timeEntry._blurredInput = null } var d = $.data(c, m); $.timeEntry._doFocus(c); var e = $.timeEntry._getSpinnerRegion(d, a); $.timeEntry._changeSpinner(d, b, e); $.timeEntry._actionSpinner(d, e); $.timeEntry._timer = null; $.timeEntry._handlingSpinner = true; var f = $.timeEntry._get(d, 'spinnerRepeat'); if (e >= 3 && f[0]) { $.timeEntry._timer = setTimeout(function () { $.timeEntry._repeatSpinner(d, e) }, f[0]); $(b).one('mouseout', $.timeEntry._releaseSpinner).one('mouseup', $.timeEntry._releaseSpinner) } }, _actionSpinner: function (a, b) { if (!a.input.val()) { $.timeEntry._parseTime(a) } switch (b) { case 0: this._setTime(a); break; case 1: this._changeField(a, -1, false); break; case 2: this._changeField(a, +1, false); break; case 3: this._adjustField(a, +1); break; case 4: this._adjustField(a, -1); break } }, _repeatSpinner: function (a, b) { if (!$.timeEntry._timer) { return } $.timeEntry._lastInput = $.timeEntry._blurredInput; this._actionSpinner(a, b); this._timer = setTimeout(function () { $.timeEntry._repeatSpinner(a, b) }, this._get(a, 'spinnerRepeat')[1]) }, _releaseSpinner: function (a) { clearTimeout($.timeEntry._timer); $.timeEntry._timer = null }, _endExpand: function (a) { $.timeEntry._timer = null; var b = $.timeEntry._getSpinnerTarget(a); var c = $.timeEntry._getInput(b); var d = $.data(c, m); $(b).remove(); d._expanded = false }, _endSpinner: function (a) { $.timeEntry._timer = null; var b = $.timeEntry._getSpinnerTarget(a); var c = $.timeEntry._getInput(b); var d = $.data(c, m); if (!$.timeEntry._isDisabledTimeEntry(c)) { $.timeEntry._changeSpinner(d, b, -1) } if ($.timeEntry._handlingSpinner) { $.timeEntry._lastInput = $.timeEntry._blurredInput } if ($.timeEntry._lastInput && $.timeEntry._handlingSpinner) { $.timeEntry._showField(d) } $.timeEntry._handlingSpinner = false }, _getSpinnerTarget: function (a) { return a.target || a.srcElement }, _getSpinnerRegion: function (a, b) { var c = this._getSpinnerTarget(b); var d = ($.browser.opera || $.browser.safari ? $.timeEntry._findPos(c) : $(c).offset()); var e = ($.browser.safari ? $.timeEntry._findScroll(c) : [document.documentElement.scrollLeft || document.body.scrollLeft, document.documentElement.scrollTop || document.body.scrollTop]); var f = this._get(a, 'spinnerIncDecOnly'); var g = (f ? 99 : b.clientX + e[0] - d.left - ($.browser.msie ? 2 : 0)); var h = b.clientY + e[1] - d.top - ($.browser.msie ? 2 : 0); var i = this._get(a, (a._expanded ? 'spinnerBigSize' : 'spinnerSize')); var j = (f ? 99 : i[0] - 1 - g); var k = i[1] - 1 - h; if (i[2] > 0 && Math.abs(g - j) <= i[2] && Math.abs(h - k) <= i[2]) { return 0 } var l = Math.min(g, h, j, k); return (l == g ? 1 : (l == j ? 2 : (l == h ? 3 : 4))) }, _changeSpinner: function (a, b, c) { $(b).css('background-position', '-' + ((c + 1) * this._get(a, (a._expanded ? 'spinnerBigSize' : 'spinnerSize'))[0]) + 'px 0px') }, _findPos: function (a) { var b = curTop = 0; if (a.offsetParent) { b = a.offsetLeft; curTop = a.offsetTop; while (a = a.offsetParent) { var c = b; b += a.offsetLeft; if (b < 0) { b = c } curTop += a.offsetTop } } return { left: b, top: curTop } }, _findScroll: function (a) { var b = false; $(a).parents().each(function () { b |= $(this).css('position') == 'fixed' }); if (b) { return [0, 0] } var c = a.scrollLeft; var d = a.scrollTop; while (a = a.parentNode) { c += a.scrollLeft || 0; d += a.scrollTop || 0 } return [c, d] }, _get: function (a, b) { return (a.options[b] != null ? a.options[b] : $.timeEntry._defaults[b]) }, _parseTime: function (a) { var b = this._extractTime(a); var c = this._get(a, 'showSeconds'); if (b) { a._selectedHour = b[0]; a._selectedMinute = b[1]; a._selectedSecond = b[2] } else { var d = this._constrainTime(a); a._selectedHour = d[0]; a._selectedMinute = d[1]; a._selectedSecond = (c ? d[2] : 0) } a._secondField = (c ? 2 : -1); a._ampmField = (this._get(a, 'show24Hours') ? -1 : (c ? 3 : 2)); a._lastChr = ''; a._field = Math.max(0, Math.min(Math.max(1, a._secondField, a._ampmField), this._get(a, 'initialField'))); if (a.input.val() != '') { this._showTime(a) } }, _extractTime: function (a, b) { b = b || a.input.val(); var c = this._get(a, 'separator'); var d = b.split(c); if (c == '' && b != '') { d[0] = b.substring(0, 2); d[1] = b.substring(2, 4); d[2] = b.substring(4, 6) } var e = this._get(a, 'ampmNames'); var f = this._get(a, 'show24Hours'); if (d.length >= 2) { var g = !f && (b.indexOf(e[0]) > -1); var h = !f && (b.indexOf(e[1]) > -1); var i = parseInt(d[0], 10); i = (isNaN(i) ? 0 : i); i = ((g || h) && i == 12 ? 0 : i) + (h ? 12 : 0); var j = parseInt(d[1], 10); j = (isNaN(j) ? 0 : j); var k = (d.length >= 3 ? parseInt(d[2], 10) : 0); k = (isNaN(k) || !this._get(a, 'showSeconds') ? 0 : k); return this._constrainTime(a, [i, j, k]) } return null }, _constrainTime: function (a, b) { var c = (b != null); if (!c) { var d = this._determineTime(a, this._get(a, 'defaultTime')) || new Date(); b = [d.getHours(), d.getMinutes(), d.getSeconds()] } var e = false; var f = this._get(a, 'timeSteps'); for (var i = 0; i < f.length; i++) { if (e) { b[i] = 0 } else if (f[i] > 1) { b[i] = Math.round(b[i] / f[i]) * f[i]; e = true } } return b }, _showTime: function (a) { var b = this._get(a, 'show24Hours'); var c = this._get(a, 'separator'); var d = (this._formatNumber(b ? a._selectedHour : ((a._selectedHour + 11) % 12) + 1) + c + this._formatNumber(a._selectedMinute) + (this._get(a, 'showSeconds') ? c + this._formatNumber(a._selectedSecond) : '') + (b ? '' : this._get(a, 'ampmPrefix') + this._get(a, 'ampmNames')[(a._selectedHour < 12 ? 0 : 1)])); this._setValue(a, d); this._showField(a) }, _showField: function (a) { var b = a.input[0]; if (a.input.is(':hidden') || $.timeEntry._lastInput != b) { return } var c = this._get(a, 'separator'); var d = c.length + 2; var e = (a._field != a._ampmField ? (a._field * d) : (a._ampmField * d) - c.length + this._get(a, 'ampmPrefix').length); var f = e + (a._field != a._ampmField ? 2 : this._get(a, 'ampmNames')[0].length); if (b.setSelectionRange) { b.setSelectionRange(e, f) } else if (b.createTextRange) { var g = b.createTextRange(); g.moveStart('character', e); g.moveEnd('character', f - a.input.val().length); g.select() } if (!b.disabled) { b.focus() } }, _formatNumber: function (a) { return (a < 10 ? '0' : '') + a }, _setValue: function (a, b) { if (b != a.input.val()) { a.input.val(b).trigger('change') } }, _changeField: function (a, b, c) { var d = (a.input.val() == '' || a._field == (b == -1 ? 0 : Math.max(1, a._secondField, a._ampmField))); if (!d) { a._field += b } this._showField(a); a._lastChr = ''; $.data(a.input[0], m, a); return (d && c) }, _adjustField: function (a, b) { if (a.input.val() == '') { b = 0 } var c = this._get(a, 'timeSteps'); this._setTime(a, new Date(0, 0, 0, a._selectedHour + (a._field == 0 ? b * c[0] : 0) + (a._field == a._ampmField ? b * 12 : 0), a._selectedMinute + (a._field == 1 ? b * c[1] : 0), a._selectedSecond + (a._field == a._secondField ? b * c[2] : 0))) }, _setTime: function (a, b) { b = this._determineTime(a, b); var c = this._constrainTime(a, b ? [b.getHours(), b.getMinutes(), b.getSeconds()] : null); b = new Date(0, 0, 0, c[0], c[1], c[2]); var b = this._normaliseTime(b); var d = this._normaliseTime(this._determineTime(a, this._get(a, 'minTime'))); var e = this._normaliseTime(this._determineTime(a, this._get(a, 'maxTime'))); b = (d && b < d ? d : (e && b > e ? e : b)); var f = this._get(a, 'beforeSetTime'); if (f) { b = f.apply(a.input[0], [this._getTimeTimeEntry(a.input[0]), b, d, e]) } a._selectedHour = b.getHours(); a._selectedMinute = b.getMinutes(); a._selectedSecond = b.getSeconds(); this._showTime(a); $.data(a.input[0], m, a) }, _normaliseTime: function (a) { if (!a) { return null } a.setFullYear(1900); a.setMonth(0); a.setDate(0); return a }, _determineTime: function (i, j) { var k = function (a) { var b = new Date(); b.setTime(b.getTime() + a * 1000); return b }; var l = function (a) { var b = $.timeEntry._extractTime(i, a); var c = new Date(); var d = (b ? b[0] : c.getHours()); var e = (b ? b[1] : c.getMinutes()); var f = (b ? b[2] : c.getSeconds()); if (!b) { var g = /([+-]?[0-9]+)\s*(s|S|m|M|h|H)?/g; var h = g.exec(a); while (h) { switch (h[2] || 's') { case 's': case 'S': f += parseInt(h[1], 10); break; case 'm': case 'M': e += parseInt(h[1], 10); break; case 'h': case 'H': d += parseInt(h[1], 10); break } h = g.exec(a) } } c = new Date(0, 0, 10, d, e, f, 0); if (/^!/.test(a)) { if (c.getDate() > 10) { c = new Date(0, 0, 10, 23, 59, 59) } else if (c.getDate() < 10) { c = new Date(0, 0, 10, 0, 0, 0) } } return c }; return (j ? (typeof j == 'string' ? l(j) : (typeof j == 'number' ? k(j) : j)) : null) }, _handleKeyPress: function (a, b) { if (b == this._get(a, 'separator')) { this._changeField(a, +1, false) } else if (b >= '0' && b <= '9') { var c = parseInt(b, 10); var d = parseInt(a._lastChr + b, 10); var e = this._get(a, 'show24Hours'); var f = (a._field != 0 ? a._selectedHour : (e ? (d < 24 ? d : c) : (d >= 1 && d <= 12 ? d : (c > 0 ? c : a._selectedHour)) % 12 + (a._selectedHour >= 12 ? 12 : 0))); var g = (a._field != 1 ? a._selectedMinute : (d < 60 ? d : c)); var h = (a._field != a._secondField ? a._selectedSecond : (d < 60 ? d : c)); var i = this._constrainTime(a, [f, g, h]); this._setTime(a, new Date(0, 0, 0, i[0], i[1], i[2])); a._lastChr = b } else if (!this._get(a, 'show24Hours')) { b = b.toLowerCase(); var j = this._get(a, 'ampmNames'); if ((b == j[0].substring(0, 1).toLowerCase() && a._selectedHour >= 12) || (b == j[1].substring(0, 1).toLowerCase() && a._selectedHour < 12)) { var k = a._field; a._field = a._ampmField; this._adjustField(a, +1); a._field = k; this._showField(a) } } } }); function extendRemove(a, b) { $.extend(a, b); for (var c in b) { if (b[c] == null) { a[c] = null } } return a } var n = ['getOffset', 'getTime', 'isDisabled']; $.fn.timeEntry = function (c) { var d = Array.prototype.slice.call(arguments, 1); if (typeof c == 'string' && $.inArray(c, n) > -1) { return $.timeEntry['_' + c + 'TimeEntry'].apply($.timeEntry, [this[0]].concat(d)) } return this.each(function () { var a = this.nodeName.toLowerCase(); if (a == 'input') { if (typeof c == 'string') { $.timeEntry['_' + c + 'TimeEntry'].apply($.timeEntry, [this].concat(d)) } else { var b = ($.fn.metadata ? $(this).metadata() : {}); $.timeEntry._connectTimeEntry(this, $.extend(b, c)) } } }) }; $.timeEntry = new TimeEntry() })(jQuery);

(function ($) {
	$.fn.listeditor = function (options) {
		
		if( !$(this).data("listeditor") ){
			var dafalutOptions = {
				validate: function(val){return true;},
				validateOnSave: function(){return true;},
				selectedClass: "selected",
				editorFiled: ".itemeditor .editField",
				editorButton: ".itemeditor .button",
				errorClass: "error",
				validationMessage: "invalid value",
				validationPlaceholder: ".itemeditor .validatorMsg",
				item: ".item",
				itemText: null,
				clearBtn: ".clearBtn",
				itemRender: null,
				isSaveOnClick : false,
				checkDuplicate : false
			};

			return ({
				_data: null,
				_itemTemplate: "",
				_listContainer: null,
				_selectedItem: -1,
				_elt: null,
				_enabled: true,
				_options: {},
				_disableEvents: false,

				methods : {
					"getValue": true,
					"setValue": true,
					"saveItem": true,
					"addItem": true,
					"removeItem": true,
					"removeNumItem": true,
					"removeSelection": true,
					"showError": true,
					"clear": true,
					"setSaveOnClick":true,
					"isDuplicate":true,
					"enable": true,
					"disableEvents": true,
					"enableEvents": true
				},

				create: function (elt, o) {
					this._elt = elt;
					this._options = o;
					this._data = [];
					this._elt.data("listeditor", this);
					if(this._options.editorButton)
						elt.find(this._options.editorButton).click((function (obj) { return function () { return obj._saveBtnHandler() }; })(this));
					if(this._options.editorFiled)
						elt.find(this._options.editorFiled).on("keyup paste blur", (function (obj) { return function () { window.setTimeout(function () { obj._validateTyped(); }, 10); }; })(this))
							.on("keydown", (function (obj) { return function (e) { return obj._checkEnter(e);}; })(this));
					if(this._options.clearBtn)
						elt.find(this._options.clearBtn).click((function (obj) { return function () { return obj._clearField() }; })(this));
					this._listContainer = elt.find(this._options.item).parent();
					this._itemTemplate = this._listContainer.html();
					this._listContainer.empty();
					this._toggleAddEnabled(false);
					this._hideError();
					return $(this);
				},

				enable: function(enableCtrl) {
					this._enabled = enableCtrl;
					if (!enableCtrl) {
						this._clearField();
						this.removeSelection();
					}
					if(this._options.editorFiled)
						this._elt.find(this._options.editorFiled).attr("disabled", !enableCtrl);
					this._listContainer.attr("disabled", !enableCtrl);
				},

				disableEvents: function() {
					this._disableEvents = true;
				},

				enableEvents: function() {
					this._disableEvents = false;
				},

				getValue: function(){
					return this._data;
				},

				setValue: function(val){
					var limited = typeof(this._options.maxLength) == "number";
					if(typeof(val) != "undefined"){
						this._selectedItem = -1;
						this._data = [];
						if( !(val instanceof Array ))
							val = Array.prototype.slice.call(arguments);

						this._listContainer.empty();
						for (var i = 0, l = val.length; i < l && ((limited && i < this._options.maxLength) || !limited); i++) {
							this._addItem(val[i], i, true);
						}

						for (; i < l; i++) {
							this._data.push(typeof(this._options.editorItemHelper) == "function" ? this._options.editorItemHelper(val[i]) : val[i]);
						}

						$(this).trigger("onlengthlimited", limited && l >= this._options.maxLength);

						this._toggleAddEnabled(false);
						this._hideError();
					}
				},

				isDuplicate: function(val,params){
					//debugger;
					if (!this._options.checkDuplicate) return false;
					for( var i=0;i<this._data.length;i++) {
						var item = this._data[i];
						if (params instanceof Array) {
							var f = true;
							for(var j=0;j<params.length;j++) 
								if (item[params[j]]!=val[params[j]]) {
									f = false;
									break;
								}
							if (f) return true;	
						}
						else {
							if (item==val) return true;
						}
						
					}					
	
					return false;
				},
				
				addItem: function(val){
					this._addItem(val, this._data.length);
					
					if (!this._disableEvents) {
						$(this).trigger("onvaluechange", [this._data]);
						$(this).trigger("onlengthlimited", this._options.maxLength && this._data.length >= this._options.maxLength);
					}
				},

				removeItem: function(){
					if (this._selectedItem >= 0){
						this.removeNumItem(this._selectedItem);
					}
				},

				removeNumItem: function (n) {
					if (typeof (this._data[n]) != "undefined") {
						this.setValue(this._data.slice(0, n).concat(this._data.slice((n >= 0 ? n : this._data.length + n) + 1)));
						this._clearField();
						$(this).trigger("onvaluechange", [this._data]);
						$(this).trigger("onlengthlimited", this._options.maxLength && this._data.length >= this._options.maxLength);
					}
				},

				clear: function(){
					this._listContainer.empty();
					this._selectedItem = -1;
					this._data = [];
					$(this).trigger("onvaluechange", [this._data]);
					$(this).trigger("onlengthlimited", false);
				},

				showError: function(){
					this._showError();
				},

				_clearField: function(){
					if(this._options.editorFiled)
						this._elt.find(this._options.editorFiled).focus().val("");
					return false;
				},

				_validateTyped: function(){
					//debugger;
					if(this._options.editorFiled){
						var val = $.trim(this._elt.find(this._options.editorFiled).val());
						//this._hideError();
						
						this._toggleAddEnabled(!!val);

						if(typeof(this._options.validate)=="function" && this._options.validate(val)===false)
							if (this._isEnabled) this._showError();
					}
				},

				_isEnabled: false,
				_toggleAddEnabled: function(isEnabled){
					if(this._options.editorButton)
						this._elt.find(this._options.editorButton).attr("disabled", !isEnabled);
					this._isEnabled = isEnabled;
				},

				_addItem: function (data, indx, disableToggleAdd) {
					var item_i;
					
					if( typeof(this._options.itemRender) == "function" ) {
						item_i = this._options.itemRender(item_i, data, this._itemTemplate);
						data = item_i.data("listeditor-item-data");
					} else {
						item_i = $(this._itemTemplate);
						data = this._setItemText(item_i, data);
					}
					
					this._data.push(data);
					
					item_i.click(
						(function (obj, i, elt) { 
							return function () {
								return obj._toggleItem( i, elt )
							}; 
						})(this, indx, item_i)
					);
					item_i.appendTo(this._listContainer);

					if (!this._disableEvents)
						$(this).trigger("onitemcreate", [item_i, data, indx]);

					if (disableToggleAdd == undefined)
						this._toggleAddEnabled(false);
				},

				_setItemText: function(item, data){
					item.data("listeditor-item-data", data);

					var itemElt = item;
					if( this._options.itemText && item.find(this._options.itemText).length > 0 )
						itemElt = item.find(this._options.itemText);

					itemElt.attr("title", data).text(data);
					return data;
				},

				_setItem: function (data, indx) {
					var itm = this._elt.find(this._options.item).eq(indx);
					data = this._setItemText(itm, data);
					this._data[indx] = data;
				},

				removeSelection: function () {
					if (this._elt) {
						this._elt.find("." + this._options.selectedClass).removeClass(this._options.selectedClass).data("listeditor-is-selected", false);
						this._selectedItem = -1;
					}
				},

				_toggleItem: function (indx, elt) {
					if (this._elt && this._enabled) {
						this._hideError();
						var isSelected = !elt.data("listeditor-is-selected");
						var data = elt.data("listeditor-item-data");
						this._elt.find("." + this._options.selectedClass).removeClass(this._options.selectedClass).data("listeditor-is-selected", false);
						elt.toggleClass(this._options.selectedClass, isSelected).data("listeditor-is-selected", isSelected);
						this._selectedItem = isSelected ? indx : -1;
						if(this._options.editorFiled)
							this._elt.find(this._options.editorFiled).val(isSelected ? data : "");
						this._toggleAddEnabled(isSelected); //my
						$(this).trigger("onitemselect",[isSelected, data]);
					}
				},

				_hideError: function(){
					if(this._options.editorFiled){
						this._elt.find(this._options.editorFiled).removeClass( this._options.errorClass ).parent().removeClass( this._options.errorClass );
						this._elt.find(this._options.validationPlaceholder).hide().empty();
					}
				},

				_showError: function(){
					if(this._options.editorFiled){
						this._elt.find(this._options.editorFiled).addClass( this._options.errorClass );//.parent().addClass( this._options.errorClass );
						this._elt.find(this._options.validationPlaceholder).show().html(typeof(this._options.validationMessage)=="function"?this._options.validationMessage():this._options.validationMessage);
					}
				},

				_checkEnter: function(e){
					
					if(e){
						if( e.which == 13 ){
							e.stopPropagation();
							e.preventDefault();
							window.setTimeout( (function(obj){ return function(){ obj._saveBtnHandler(); };})(this), 0);
							return false;
						}
					}
				},

				setSaveOnClick:function (val) {
					this._options.isSaveOnClick = val;
				},
				
				_saveBtnHandler: function () {
					var val = $.trim(this._elt.find(this._options.editorFiled).val());

					if(this._isEnabled){
						this._hideError();
						if (val) {
							var isValid = this._options.validateOnSave(val);
							if( isValid === true )
								this.saveItem(val);
							else if(isValid === false)
								this._showError(val);
						}
					}
				},
				
				saveItem: function(val){
					this._hideError();
						
					if (this._options.isSaveOnClick) {
						if (this._selectedItem >= 0)
							this._setItem(val, this._selectedItem);
						else
							this._addItem(val, this._data.length);

						if(this._options.editorFiled)
							this._elt.find(this._options.editorFiled).val("");
						this._selectedItem = -1;
					}



					this._elt.find("."+this._options.selectedClass).removeClass(this._options.selectedClass)
						.data("listeditor-is-selected", false);

					if (this._options.isSaveOnClick) {
						$(this).trigger("onvaluechange", [this._data]);
						$(this).trigger("onlengthlimited", this._options.maxLength && this._data.length >= this._options.maxLength);
					}
					else
						$(this).trigger("onotheraction", [this._data]);
					
				}
				
			}).create($(this), $.extend(dafalutOptions, options));
		}else{
			var method = arguments[0];
			var editor = $(this).data("listeditor");
			if( typeof (method )!= "undefined" && editor.methods[method] ){
				return editor[method].apply(editor, Array.prototype.slice.call( arguments, 1 ) );
			}
		}

	};
})(jQuery);

(function ($) {
	$.fn.threeStateCheckbox = function (options) {
		if( !$(this).data("threeStateCheckbox") ){
			return ({
				methods : {
					"checkPartial": true,
					"checkAll": true,
					"uncheck": true
				},

				state: { "none": 0, "all": 1, "partial": 2 },
				_currState: 0,
				_elt: null,

				init: function($this, o){
					$this.find("input[type=checkbox]")
						.click( (function (obj, $t) { return function (e) { obj._selectAllClick(e,$t) }; })(this, $this) )
						.hover(
							(function (obj, $t) { return function (e) { obj._over($t) }; })(this, $this),
							(function (obj, $t) { return function (e) { obj._out($t) }; })(this, $this)
						);
					this._elt = $this;
					$this.data("threeStateCheckbox", this);
					return $this;
				},
				
				uncheck: function(cancelEvent){ this._setCheckState(this._elt, this.state.none, cancelEvent); },
				checkPartial: function(cancelEvent){ this._setCheckState(this._elt, this.state.partial, cancelEvent); },
				checkAll: function(cancelEvent){ this._setCheckState(this._elt, this.state.all, cancelEvent); },

				_over: function($this){ $this.addClass("over"); },
				_out: function($this){ $this.removeClass("over"); },

				_selectAllClick: function(e, $this){
					/// <param name="e" type="jQuery.Event">Description</param>
					e.stopPropagation();
					var isAllChecked = $this.find("input[type=checkbox]").is(":checked");
					this._setCheckState($this, isAllChecked?this.state.all:this.state.none);
				},
				_setCheckState: function($this, state, cancelEvent){
					var statesClasses = ["checkednone", "checked", "checkedpartial"];
					var stateClass = statesClasses[state];
					$this.removeClass(statesClasses.slice(1).join(" "));
					$this.addClass( stateClass );

					$this.find("input[type=checkbox]").attr("checked", state==this.state.all );
					this._currState = state;
					if(!cancelEvent)
						$this.trigger("on"+stateClass, [$this]);
				}
			}).init($(this), options);
		}else{
			var method = arguments[0];
			var editor = $(this).data("threeStateCheckbox");
			if( typeof (method )!= "undefined" && editor.methods[method] ){
				return editor[method].apply(editor, Array.prototype.slice.call( arguments, 1 ) );
			}
		}
	};
})(jQuery);

(function ($) {
	$.fn.numberselect = function (options) {
		if( !$(this).data("numberselect") ){

			return ({
				_usewheel: false,

				methods : {
					"setValue": true
				},

				_options: {
					min: 0,
					max: Number.MAX_VALUE,
					step: 1,
					defaultValue: 0
				},

				init: function($this, o){
					if( $this.is("input[type=text]") ){
						this._options = $.extend( this._options, o||{} );

						$this.wrap("<span class='numberselect'></span>");
						$this.parent().append("<span class='btnUp'></span><span class='btnDown'></span>");
						$this.data("numberselect", this)
							.focus((function(obj, elt){ return function(){ if( !elt.is("[disabled]") ){ obj._usewheel = true; } } })(this, $this))
							.blur((function(obj, elt){ return function(e){ return obj._focusOut(e, elt); } })(this, $this))
							.keydown((function(obj, elt){ return function(e){ return obj._validate(e, elt); } })(this, $this))
							.on("paste", (function(obj, elt){ return function(e){ return obj._paste(e, elt); } })(this, $this));
						$this.parent().find(".btnUp").click((function(obj, elt){ return function(){ obj._increase(1, elt); } })(this, $this));
						$this.parent().find(".btnDown").click((function(obj, elt){ return function(){ obj._increase(-1, elt) } })(this, $this));
						$this.val(this._options.defaultValue||0);
						$(document).mousewheel((function(obj, elt){ return function(e, delta){ obj._wheelhandler(delta, elt) } })(this, $this));
					}
					return $this;
				},
				
				_paste: function(e, elt) {
					var val = e.originalEvent.clipboardData.getData("text");
					
					if(isNaN(val))
						return false;

					if (+val < this._options.min || +val > this._options.max)
						return false;

					return true;
				},

				_focusOut: function(e, elt){
					if( !elt.is("[disabled]") ){
						var val = $.trim(elt.val()) * 1;
						if( val < this._options.min || val != val ){
							elt.val(this._options.min);
							return false;
						}
						this._usewheel = false;
					}
				},

				_validate: function(e, elt){
					if( elt.is("[disabled]") )
						return;
					console.log(e.which);
					if (e && typeof (e.which) != "undefined" && !({ 8: true, 9: true, 13: true, 37: true, 39: true, 46: true })[e.which]) {
						var start = this._getSelectionStart(elt[0]);
						var end = this._getSelectionEnd(elt[0]);
						var num = String.fromCharCode(e.which > 90 ? e.which - 48 : e.which);
						var str = elt.val();
						var val = $.trim(str.substring(0, start) + num + str.substring(end)) * 1;
						console.log(start + " - " + num + " - " +val + " - " + end);

						if( val > this._options.max || val != val )
							return false;
					}
				},

				_getSelectionStart: function (elt) {
					if (elt.createTextRange) {
						var r = document.selection.createRange().duplicate();
						r.moveEnd('character', elt.value.length);
						return !r.text?elt.value.length:elt.value.lastIndexOf(r.text);
					} else return elt.selectionStart;
				},

				_getSelectionEnd: function (elt) {
					if (elt.createTextRange) {
						var r = document.selection.createRange().duplicate();
						r.moveStart('character', -elt.value.length);
						return r.text.length;
					} else return elt.selectionEnd;
				},

				_wheelhandler: function(delta, elt){
					if( elt.is("[disabled]") )
						return;
					if(this._usewheel)
						this._increase(delta>0?1:-1, elt);
				},

				_increase: function (delta, elt) {
					if( elt.is("[disabled]") )
						return;
					if (this._evtTimer)
						window.clearTimeout(this._evtTimer)
					this._evtTimer = null;

					var val = elt.val()*1 + delta*this._options.step;
					val = val < this._options.min?this._options.min:val;
					val = val > this._options.max?this._options.max:val;
					elt.val(val);

					this._evtTimer = window.setTimeout((function(obj, v, e){ return function(){ obj._notifyChange(v, e) } })(this, val, elt), 500);
				},

				_notifyChange: function(val, elt){
					if( elt.is("[disabled]") )
						return;
					elt.trigger("onchange", [val]);
				}
			}).init($(this), options);
		}else{
			var method = arguments[0];
			var editor = $(this).data("numberselect");
			if( typeof (method )!= "undefined" && editor.methods[method] ){
				return editor[method].apply(editor, Array.prototype.slice.call( arguments, 1 ) );
			}
		}
	};
})(jQuery);

(function ($) {
	var timeformats = { "12h": 12, "24h": 24 },
		defaultOptions = {
			min: 0,
			max: Number.MAX_VALUE,
			step: 1,
			defaultValue: null,
			value: null,
			timeFormat: "24h",
			amText: "am",
			pmText: "pm"
		},
		meridVal = {
			"am": 1,
			"pm": 2
		}
		methods = {
			"setValue": true,
			"getValue": true,
			"getHours": true,
			"setHours": true,
			"getMinutes": true,
			"setMinutes": true,
			"show": true,
			"hide": true,
			"toggle": true
		};

	$.fn.timeselect = function (options) {
		if( !$(this).data("timeselect") ){

			return ({
				_usewheel: false,

				_options: {},

				show: function(){
					this._elt.closest(".timeselect").show(0);
				},
				hide: function(){
					this._elt.closest(".timeselect").show(0);
				},
				toggle: function(b){
					this._elt.closest(".timeselect").toggle(b);
				},

				init: function($this, o){
					if( $this.is("input[type=text]") ){
						this._options = $.extend( defaultOptions, o||{} );
						this._elt = $this;
						$this.wrap("<span class='timeselect'></span>");
						$ctrl = $this.closest(".timeselect");

						$ctrl.css({ "position": "relative", "overflow": "hidden" })
							.append("<input type='text' class='input hours' /><span class='colon'>:</span><input type='text' class='input minutes' /><span class='meridSign'>am/pm</span><select class=\"meridSelect\"><option class=\"itemAm str-local-ResTimeSelectAM\" value=\"1\"></option><option class=\"itemPm str-local-ResTimeSelectPM\" value=\"2\"></option></select><span class='btnUp'></span><span class='btnDown'></span>");

						$this.data("timeselect", this).css({
							"position": "absolute",
							"left": 0,
							"top": "100%"
						}).focus((function(obj){ return function(){ obj._focus(); } })(this));
						
						$ctrl.focus((function(obj){ return function(){ if( !elt.is("[disabled]") ){ obj._usewheel = true; } } })(this, $this))
							.blur((function(obj){ return function(){ if( !elt.is("[disabled]") ){ obj._usewheel = false; } } })(this, $this))
							.keyup((function(obj, elt){ return function(e){ return obj._keypress(e, obj, elt); } })(this, $this));

						$ctrl.find(".hours").on("click",(function (obj, t) { return function () { obj._setActiveTimePart(t); } })(this, this._timePart.Hours))
							.focusin((function (obj, t) { return function () { obj._setActiveTimePart(t); } })(this, this._timePart.Hours))
							.focusout((function(obj){ return function(){ obj._usewheel = false; } })(this));
						$ctrl.find(".minutes").on("click",(function (obj, t) { return function () { obj._setActiveTimePart(t); } })(this, this._timePart.Minutes))
							.focusin((function (obj, t) { return function () { obj._setActiveTimePart(t); } })(this, this._timePart.Minutes))
							.focusout((function(obj){ return function(){ obj._usewheel = false; } })(this));;

						$ctrl.find(".btnUp").on("click",(function(obj, elt){ return function(){ obj._increase(1, elt); } })(this, $this));
						$ctrl.find(".btnDown").on("click",(function(obj, elt){ return function(){ obj._increase(-1, elt) } })(this, $this));
						$ctrl.find(".meridSelect").on("change",(function(obj, elt, selct){ return function(){ obj._meridChange(selct, elt) } })(this, $this, $ctrl.find(".meridSelect")));
						
						$(document).mousewheel((function (obj, elt) { return function (e, delta) { obj._wheelhandler(delta, elt) } })(this, $this));

						this._parseVal();

						this._applyValue();
					}
					return $this;
				},
				
				_parseVal: function(val){
					val = val || this._options.value || this._options.defaultValue || new Date();
					if( val instanceof Date ){
						this.setHours(val.getHours());
						this.setMinutes(val.getMinutes());
					}else if( val instanceof Array && val.length > 0 ){
						this.setHours(val[0]);
						if(val.length>1)
							this.setMinutes(val[1]);
					}else if( (/[0-9]{1,2}:[0-9]{1,2}/).test(val) ){
						var val_s = (val+"").split(":");
						if(val_s.length>0){
							this.setHours(val_s[0]);
							if(val_s.length>1)
								this.setMinutes(val_s[1]);
						}
					}else if( val.hasOwnProperty("hours") && val.hasOwnProperty("minutes") ){
						this.setHours(val.hours);
						this.setMinutes(val.minutes);
					}else{
						var d = new Date();
						this.setHours(d.getHours());
						this.setMinutes(d.getMinutes());
					}
				},

				_keypress: function (e, obj, elt) {
					if (e.which == 9)
						return;
					if (e.which == 13)
						return false;
					var num = String.fromCharCode(e.which > 90 ? (e.which - 48) : e.which);
					try { console.log("Time(" + e.which + "): " + num) } catch (e) { };
					if (([1, 1, 1, 1, 1, 1, 1, 1, 1, 1])[num]) {
						if (this._evtTimer)
							window.clearTimeout(this._evtTimer);
						this._evtTimer = null;
						num = num * 1;
						var type = this._activeTimePart == this._timePart.Hours ? "hours" : "minutes";
						var maxVal = this._activeTimePart == this._timePart.Hours ? 23 : 59;
						var currVal = this.value[type];

						var val = currVal*10 + num;
						if (val > maxVal)
							val = num;

						try { console.log("Time(" + e.which + "): val=" + val) } catch (e) { };
						this.value[type] = val;
						
						this._evtTimer = window.setTimeout((function (obj, v, el) { return function () { obj._notifyChange(v, el) } })(this, val, elt), 500);
					}
					this._applyValue();
					window.setTimeout((function (obj) { return function () { obj._select(); }; })(this), 0);
				},

				_select: function () {
					var type = this._activeTimePart == this._timePart.Hours ? ".hours" : ".minutes";
					var elt = this._elt.closest(".timeselect").find(type)[0];
					try { console.log("Time: elt=" + elt) } catch (e) { };

					if (elt.setSelectionRange) {
						elt.focus();
						elt.setSelectionRange(0,2);
					} else if (elt.createTextRange) {
						var range = elt.createTextRange();
						range.collapse(true);
						range.moveEnd('character', 2);
						range.moveStart('character', 0);
						range.select();
					}
					
				},

				_focus: function(){
					//this._elt.closest(".timeselect").find(".hours").focus()
					return false;
				},

				_activeTimePart: 0,
				_timePart: { Hours: 0, Minutes: 1 },
				value: null,
				_evtTimer: null,
				_elt: null,

				_setActiveTimePart: function(t){
					this._usewheel = true;
					this._activeTimePart = t;
					try { console.log("Time: focusin") } catch (e) { };
					window.setTimeout((function (obj) { return function () { obj._select(); }; })(this), 0);
				},

				getHours: function () {
					return this.value ? this.value.hours : null;
				},

				getMinutes: function () {
					return this.value ? this.value.minutes : null;
				},

				setHours: function (h) {
					this.value = this.value || { hours: 0, minutes: 0 };
					h = h*1;
					if(!isNaN(h)){
						this.value.hours = h < 0 ? 0 : (h > 23 ? 23 : h);
						this._applyValue();
					}
				},

				setMinutes: function (m) {
					this.value = this.value || { hours: 0, minutes: 0 };
					m = m*1;
					if(!isNaN(m)){
						this.value.minutes = m < 0 ? 0 : (m > 59 ? 59 : m);
						this._applyValue();
					}
				},

				_meridChange: function(select){
					this.value = this.value || { hours: 0, minutes: 0 };
					var tf = timeformats[(this._options.timeFormat||"24h").toLowerCase()]||timeformats["24h"],
						h = this.value.hours % tf;
					if( +$.trim(select.val()||"1") == meridVal.pm )
						h = tf + h;
					this.setHours(h);
				},

				setValue: function(val){
					this._parseVal(val);
				},

				getValue: function () {
					return this.value ? this.value.hours + ":" + this.value.minutes : "00:00";
				},

				_applyValue: function () {
					var elt = this._elt,
						tf = timeformats[(this._options.timeFormat||"24h").toLowerCase()]||timeformats["24h"],
						isAmPm = tf == timeformats["12h"],
						h = this.value.hours % (tf),
						isPm = this.value.hours >= tf,
						h = (isAmPm && h==0)?12:h,
						hs = ("00" + h).slice(-2),
						ms = ("00" + this.value.minutes).slice(-2),
						tSelect = elt.closest(".timeselect");

					try { console.log("Time: Format: "+this._options.timeFormat+": hours=" + this.value.hours + ", hs=" + hs + (isAmPm?(isPm?"PM":"AM"):"")) } catch (e) { };

					tSelect.find(".hours").val(hs);
					tSelect.find(".minutes").val(ms);
					tSelect.find(".meridSign")[isAmPm?"show":"hide"]().html( isPm?this._options.pmText:this._options.amText );
					
					var meridSelect = tSelect.find(".meridSelect").hide();
					if(isAmPm)
						meridSelect.show().css({"width":tSelect.find(".meridSign").width()}).val( meridVal[isPm?"pm":"am"] );

					elt.val(this.getValue());
				},

				_wheelhandler: function(delta, elt){
					if( elt.is("[disabled]") )
						return;
					if(this._usewheel)
						this._increase(delta > 0 ? 1 : -1, elt);
				},

				_increase: function (delta, elt) {
					if( elt.is("[disabled]") )
						return;
					if (this._evtTimer)
						window.clearTimeout(this._evtTimer)
					this._evtTimer = null;

					var type = this._activeTimePart == this._timePart.Hours ? "hours" : "minutes";
					var maxVal = this._activeTimePart == this._timePart.Hours ? 23 : 59;
					var currVal = this.value[type];
					var val = currVal + delta;
					val = val >= 0 ? (val <= maxVal ? val : maxVal) : 0;
					this.value[type] = val;
					this._applyValue();
					window.setTimeout((function (obj) { return function () { obj._select(); }; })(this), 0);

					this._evtTimer = window.setTimeout((function(obj, v, e){ return function(){ obj._notifyChange(v, e) } })(this, val, elt), 500);
				},

				_notifyChange: function(val, elt){
					if( elt.is("[disabled]") )
						return;
					elt.trigger("onchange", [val]);
				}
			}).init($(this), options);
		}else{
			var method = arguments[0];
			var editor = $(this).data("timeselect");
			if( typeof (method )!= "undefined" && methods[method] ){
				return editor[method].apply(editor, Array.prototype.slice.call( arguments, 1 ) );
			}
		}
	};
	$.timeselect = {
		setDefaults : function( o ){
			defaultOptions = $.extend(defaultOptions, o);
		}
	};
})(jQuery);

(function($){
	$.fn.ldapSearch = function(options){
		if( !$(this).data("ldapSearch") ){
			var defaultOptions = {
				"minLength": 0,
				"delay" :500
			};

			return ({
				_data : null,
				_response : null,
				_elt: null,
				_options: {},

				methods : {
					"setSearchResult": true
				},

				create: function(elt, o){
					if (elt) {
						var Delegate = KL.Utils.Delegate;
						this._response = null;
						this._elt = elt;
						this._options = o;

						elt.autocomplete({
							"minLength": this._options.minLength,
							"delay": this._options.delay,
							"source": (function(obj){ return function(a,b){ return obj._startSearch(a,b); } })(this),
							"select": (function(obj){ return function(a,b){ return obj._itemSelect(a,b); } })(this)
						}).data("autocomplete")._renderItem = (function(obj){ return function(a,b){ return obj._renderAutocompleteItem(a,b); } })(this);

						elt.data( "ldapSearch", this );
					}
					return $(this);
				},

				_startSearch : function (request, response) {
					this._elt.autocomplete("close");
					this._response = response;
					$(this).trigger("onsearchstart", [request.term]);
				},

				_itemSelect : function (e, ui) {
					this._elt.val(ui.item.label);
					$(this).trigger( "onitemselected", [ui.item] );
				},

				setSearchResult : function (data) {
					if (this._response != null && data && data.length > 0) {
						this._response($.map(data, KL.Utils.Delegate.create(this, function (item) {
							return {
								"label": item.attributes ? this._getLdapString(item.attributes) : "-",
								"value": item["entryid"],
								"itemData": item
							};
						})));
						this._response = null;
					}
				},

				_renderAutocompleteItem : function (ul, item) {
					var className = (["itemUser", "itemGroup"])[item.itemData.objectType] || "itemUnknown";

					return $(KL.Utils.StringHelper.format("<li class='ldapSearchItem {1}'><a>{0}</a></li>", item.label, className))
						.data("item.autocomplete", item)
						.appendTo(ul.addClass("ldapSearchResult"));
				},

				_getLdapString : function (data) {

					var result = "";
					for (var i = 0, l = data.length; i < l; i++) {
						if (data[i]["attribute"] != "objectClass")
							result = result + data[i]["values"].join(", ") + "   ";
					}

					return result;
				}
		
			}).create( $(this), $.extend(defaultOptions, options) );
		}else{
			var method = arguments[0];
			var editor = $(this).data("ldapSearch");
			if( typeof (method )!= "undefined" && editor.methods[method] ){
				return editor[method].apply(editor, Array.prototype.slice.call( arguments, 1 ) );
			}
		}
	};
})(jQuery);

(function ($) {
	$.fn.dropdown = function (options) {
		if (!$(this).data("dropdown")) {
			var defaultOptions = {
				"minLength": 0,
				"delay": 500
			};

			return ({
				_elt: null,
				_options: {},
				_val: null,
				_hidetimer: null,

				methods: {
					"setValue": true,
					"getValue": true
				},

				setValue: function(val){
					var $this = this;
					this._elt.find( "ul a" ).each(function(){
						if( $(this).data("dropdown-value") == val )
							$this._itemSelect($(this));
					});
				},

				getValue: function(){
					return this._val;
				},

				create: function (elt, o) {
					if (elt) {
						var Delegate = KL.Utils.Delegate;
						this._response = null;
						var newElt = $('<div class="dropdown"></div>').insertBefore(elt.hide()).append('<a href="#"><span class="left"><span class="right"><span class="center"><span class="selectedText"></span></span></span></span></a><ul></ul>');
						this._options = o;

						var ul = newElt.find("ul");
						var $this = this;

						var selected = null;

						elt.find("option").each(function(){
							var option = $(this);
							var item = $("<li><a></a></li>").appendTo(ul).find("a");
							selected = selected || item;
							item.attr({
									"class": option.attr("class")
								})
								.data("dropdown-value", option.attr("value"))
								.hover(
									function(){ $this._itemOver( $(this), true ) },
									function(){ $this._itemOver( $(this), false) }
								)
								.keyup(function(e){ $this._keyHandler(e) })
								.click(function(e){ $this._itemSelect( $(this), e ); $this._keep = false; $this._forcehide(); })
								.text( option.text() );
							if( option.is(":selected") )
								selected = item;
						});
						newElt.focusin(function(e){ $this._focus(e); })
							.focusout(function(e){ $this._blur(e); })
							.find("a:eq(0)")
							.keyup(function(e){ $this._keyHandler(e) })
							.click( function(e){ $this._showddown(e); } );
						this._elt = newElt;

						this._itemSelect( selected );
						this._forcehide();
						elt.data("dropdown", this);
					}
					return $(this);
				},

				_focus: function(e){
					this._elt.find("a:eq(0)").addClass( "focused" );
					this._keepOpen(e);
				},

				_blur: function(e){
					this._elt.find("a:eq(0)").removeClass( "focused" );
					this._hideddown(e);
				},

				_keyHandler: function(event){
					if( event.which == 40 ){
						if( event ){
							event.stopPropagation();
							event.preventDefault();
						}
						var item = this._elt.find( ".itemSelected" ).closest("li").next().find("a");
						if( item.length == 0 ){
							item = this._elt.find( "ul li:eq(0) a" );
						}
						this._itemSelect(item, event);
					}else if(event.which == 38){
						if( event ){
							event.stopPropagation();
							event.preventDefault();
						}
						var item = this._elt.find( ".itemSelected" ).closest("li").prev().find("a");
						if( item.length != 0 )
							this._itemSelect(item, event);
					}
				},

				_itemOver: function(item, isOver){
					this._elt.find( ".itemOver" ).removeClass("itemOver");
					if(isOver)
						item.addClass("itemOver");
				},

				_itemSelect: function(item, event){
					/// <param name="event" type="jQuery.Event">the event</param>
					if( event ){
						event.stopPropagation();
						event.preventDefault();
					}
					this._elt.find( ".itemSelected" ).removeClass("itemSelected");
					this._elt.find(".selectedText")
						.attr({"class": "selectedText " + item.attr("class")})
						.text( item.text() );
					this._val = item.data("dropdown-value");
					item.addClass("itemSelected");
					$( this ).trigger("onitemselect", [this, this._val]);
				},

				_showddown: function(event){
					if( event ){
						event.stopPropagation();
						event.preventDefault();
					}
					this._elt.find("a:eq(0)").focus();
					this._elt.find("ul").show();
					this._keep = false;
				},

				_keepOpen: function(){
					if( this._hidetimer )
						window.clearTimeout( this._hidetimer );
					this._keep = true;
				},

				_hideddown: function(){
					if( this._hidetimer )
						window.clearTimeout( this._hidetimer );
					var $this = this;
					this._keep = false;
					this._hidetimer = window.setTimeout( function(){ $this._forcehide(); }, 500 );
				},

				_forcehide: function(){
					var $this = this;
					if(!this._keep){
						window.setTimeout( function(){ 
							$this._elt.find("ul").hide();
						}, 10 );
					}
				}

			}).create($(this), $.extend(defaultOptions, options));
		} else {
			var method = arguments[0];
			var editor = $(this).data("dropdown");
			if (typeof (method) != "undefined" && editor.methods[method]) {
				return editor[method].apply(editor, Array.prototype.slice.call(arguments, 1));
			}
		}
	};
})(jQuery);

(function ($) {
	$.fn.barchart = function (options) {
		if (!$(this).data("barchart")) {
			var defaultOptions = {
				"legendSeparator": "&nbsp;|&nbsp;",
				"layout": "vertical",
				"ySegments": 0
			};

			return ({
				_elt: null,
				_options: {},
				_val: null,
				_bar: "",
				_cssProps: {
					"vertical": "height",
					"horisontal": "width",
					"default": "height"
				},
				_barWideCssProps: {
					"horisontal": "height",
					"vertical": "width",
					"default": "width"
				},
				_barZeroRullerClass: {
					"horisontal": "zeroVRuler",
					"vertical": "zeroHRuler",
					"default": "zeroHRuler"
				},
				_barRullerClass: {
					"horisontal": "chartVRuler",
					"vertical": "chartHRuler",
					"default": "chartHRuler"
				},
				_rullerAddMethod: {
					"horisontal": "appendTo",
					"vertical": "prependTo",
					"default": "prependTo"
				},
					

				methods: {
					"setData": true,
					"getData": true
				},

				setData: function (val) {
					this._val = val;
					this._buidChart(val);
				},

				getData: function () {
					return this._val;
				},

				create: function (elt, o) {
					this._options = o;
					if (elt) {
						var Delegate = KL.Utils.Delegate;
						
						this._bar = elt.find(".barsWrap").html();
						elt.find(".barsWrap").empty();

						this._elt = elt;

						elt.data("barchart", this);
					}
					return $(this);
				},

				_makeRullers: function (max, count) {
					if (this._elt) {
						count = count || 1;
						var period = Math.ceil( max / count );

						if (period * 1 != period * 1)
							period = 0;
						var rullerSize = 100/count;
						var sizeCssProp = this._cssProps[this._options.layout] || this._cssProps["default"];
						var wrap = this._elt.find(".rullersWrap").empty();
						var StringHelper = KL.Utils.StringHelper;
						var rullerTpl = '<div class="{1}"><div class="rullerText">{0}</div></div>';
						var zRullerClass = this._barZeroRullerClass[this._options.layout] || this._barZeroRullerClass["default"];
						var rullerClass = this._barRullerClass[this._options.layout] || this._barRullerClass["default"]; this._barRullerClass[this._options.layout] || this._barRullerClass["default"];
						var rullerAddMethod = this._rullerAddMethod[this._options.layout] || this._rullerAddMethod["default"];

						$(StringHelper.format('<div class="{1}"><div class="rullerText">0</div></div>', period * i, zRullerClass))[rullerAddMethod](wrap);

						for (var i = 1; i <= count; i++)
							$(StringHelper.format(rullerTpl, (period * i || i), rullerClass))[rullerAddMethod](wrap).css(sizeCssProp, rullerSize+"%");
					}
				},

				_makeLegend: function( data ){
					var legend = [];
					for( var i=0, l=data.length; i<l; i++ ){
						legend.push( '<span class="color'+i+'">'+data[i]+'</span>' );
					}
					return legend.join(this._options.legendSeparator)
				},

				_makeSubBars: function(data, parent ){
					var bars = [];
					var wrap = parent.find('.barFill');
					var summ = 0;
					var cssProp = this._cssProps[this._options.layout] || this._cssProps["default"];
					
					for( var i=0, l=data.length; i<l; i++ ){
						bars.push($('<div class="subBar color'+i+'"></div>').prependTo(wrap));
						if( data[i]*1 == data[i]*1 )
							summ += data[i];
					}

					if( summ > 0 ){
						for( var i=0, l=data.length; i<l; i++ )
							bars[i].css(cssProp, ((100 * data[i] / summ)||.001) + "%").attr("title", data[i]);
					}
					bars = null;
					return summ;
				},

				_buidChart: function (data) {
					if (this._elt) {
						var title = data.title;
						this._options.layout = data.layout || "vertical";

						if( data.legend )
							title = this._makeLegend(data.legend);
						this._elt.find(".legend").toggle(!!title).html(title);

						if (data.data) {
							var cssProp = this._cssProps[this._options.layout] || this._cssProps["default"];
							var barWideCss = this._barWideCssProps[this._options.layout] || this._barWideCssProps["default"];
							var barsWrap = this._elt.find(".barsWrap").empty();
							var items = [];
							var values = [];
							var summ = null;

							for( var i=0, l=data.data.length, w=100/l; i<l; i++ ){
								var item = data.data[i];
								if( item !== null && typeof(item) != 'undefined' ){
									var val = (typeof (item.value) != "undefined" && item.value !== null) ? item.value : item;
									var lbl = (typeof (item.label) != "undefined" && item.label !== null) ? item.label : item;
									var elt_i = $(this._bar).appendTo(barsWrap).css(barWideCss, w + "%");
									var fillWrap = elt_i.find(".barFill");
									if (val instanceof Array) {
										val = this._makeSubBars( val, elt_i );
										summ = (summ + val) || val;
									} else {
										if (val * 1 == val * 1)
											summ = (summ + val) || val;
										fillWrap.addClass("color" + i);
									}
									fillWrap.css(cssProp, (val||.001) + "%").find(".barValue").html(lbl);
									items.push(elt_i);
									values.push(val);
								}
							}

							summ = data.max || Math.max.apply(Math, values);
							if (summ != null) {
								var approxLevel = Math.pow(10, Math.floor(Math.log(summ) / Math.log(10)));
								approxLevel = approxLevel<10?10:approxLevel;

								var _ySegments = summ <= 10? 2: this._options.ySegments;
								var max = approxLevel * _ySegments* Math.ceil(summ / (approxLevel*_ySegments) );
								if( max/summ > 2 && max > 10 )
									max = Math.ceil(approxLevel * _ySegments* Math.ceil(2*summ / (approxLevel*_ySegments) )/2);


								
								//if( (max/_ySegments)%5 )
								//	max = 5 * _ySegments * Math.ceil(max / (5*_ySegments));
								
								this._makeRullers(max, _ySegments);
								
								for( var i=0, l=data.data.length; i<l; i++ ){
									items[i].find(".barFill").css(cssProp, ((100 * values[i] / max)||.001) + "%").attr( "title", values[i] );
								}
							}
						}
					}
				}

			}).create($(this), $.extend(defaultOptions, options));
		} else {
			var method = arguments[0];
			var chart = $(this).data("barchart");
			if (typeof (method) != "undefined" && chart.methods[method]) {
				return chart[method].apply(chart, Array.prototype.slice.call(arguments, 1));
			}
		}
	};
})(jQuery);

(function ($) {
//	$.fn.checkboxstyling = function () {
//		$(this).on("click", null, null, function(e) {
		$("label.checkbox>input[type='checkbox']").live("change", function(e) {
			var elt = $(e.target);
			if (elt.is(":checked"))
				elt.closest("label.checkbox").addClass("checked");
			else
				elt.closest("label.checkbox").removeClass("checked");
		});
//	};
})(jQuery);